-- $Header:   //ARCHIVE/Archive/DB/sql/OwnedEventInProgress.sqv   1.0   Apr 06 2001 05:46:50   BRENT  $
--
-- OwnedEventInProgress.sql
--
-- Module Description
--   Defines the OwnedEventInProgress table. This is a worktable used by the 
--   ownership stored procedures for recording which events are owned by which
--   notes prior to the final _CommitNote call.
--
--   When a note is committed or cancelled the records are removed from this
--   table, so there should never be any rows in this table for very long.
--   It is NOT recommended to put any kind of automatic deletion of old records
--   on this table since anything left in here could indicate a problem that
--   should be addressed.

USE Object
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'OwnedEventInProgress' AND type = 'U')
BEGIN
  PRINT 'Creating table OwnedEventInProgress'
  EXEC("
  CREATE TABLE OwnedEventInProgress
  (
    eventcid      ClassID  NOT NULL,
    eventid       ObjID  NOT NULL,
    noteid        ObjID  NOT NULL,
    rowno         INT NOT NULL        IDENTITY,
    parentcid     ClassID  NOT NULL,
    parentid      ObjID  NOT NULL,
    ctime         DATETIME NOT NULL   CONSTRAINT OwnedEventInProgress_ctime_DF DEFAULT CURRENT_TIMESTAMP
  )
  ALTER TABLE OwnedEventInProgress ADD CONSTRAINT
    OwnedEventInProgress_PK PRIMARY KEY (noteid, eventcid, eventid, parentcid, parentid, rowno)
  ALTER TABLE OwnedEventInProgress ADD CONSTRAINT
    OwnedEventInProgress_ctime_UQ UNIQUE (ctime, rowno)
  ")
END
