-- $Header:   //ARCHIVE/Archive/DB/sql/Process_SA390_Exception.sqv   1.1   Mar 20 2001 11:04:18   BRENT  $
--
-- Process_SA390_Exception.sql
--

USE Object
GO


---------------------------------------------------------------------------------------------------
-- Process_SA390_Exception
PRINT 'Creating procedure Process_SA390_Exception'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'Process_SA390_Exception' AND type = 'P')
  DROP PROCEDURE Process_SA390_Exception
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/Process_SA390_Exception.sqv  $
-- $Revision:   1.1  $
-- $Date:   Mar 20 2001 11:04:18  $
-- $Author:   BRENT  $
--
-- Creates a state change message on a resource managed by SA/390.
--
-- @RESOURCE_TYPE is mapped to a TM390 object class id using the SA390ResourceTypeMap
-- table. If a mapping cannot be established then the resource is considered to be a
-- started task (cid=STC).
--
-- If @EVENT_TEXT is NULL the event is discarded
--
CREATE PROCEDURE Process_SA390_Exception
  @SourceCID          ClassID,
  @SourceID           ObjID,
  @FormatType         SMALLINT,
  @ActionType         SMALLINT,
  @DATE_TIME          VARCHAR(30) = NULL,
  @SEQ_NO             VARCHAR(50) = NULL,
  @SMFID              VARCHAR(8)  = NULL,
  @RESOURCE_TYPE      VARCHAR(30) = NULL,
  @OBJECT_NAME        VARCHAR(54) = NULL,
  @EVENT_TYPE         VARCHAR(40) = NULL,
  @EVENT_STATE        VARCHAR(40) = NULL,
  @APPID              VARCHAR(255) = NULL,   --- VARCHAR(16) for new Pump shipped in MRD 2000 (TBSM 1.1)
  @WKSTNID            VARCHAR(8)  = NULL,
  @IA                 VARCHAR(16) = NULL,
  @EVENT_TEXT         VARCHAR(255)= NULL
AS
  DECLARE @r INT, @datetime DATETIME

--  RAISERROR('Process_SA390_Exception @SourceCID=%s, @SourceID=%d, @FormatType=%d, @ActionType=%d', 0, 1,
--            @SourceCID, @SourceID, @FormatType, @ActionType)

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION Process_SA390_Exception
  ELSE
    SAVE TRANSACTION Process_SA390_Exception

  -- Special case a literal 'NULL' value being passed into the parameters
  -- We interpret this like a 'real' null.
  IF @EVENT_TEXT = 'NULL' SELECT @EVENT_TEXT = NULL
  IF @EVENT_STATE = 'NULL' SELECT @EVENT_STATE = NULL

  -- Check if receiving data from a pre-MRD 2000 (TBSM 1.1) pump
  --
  -- Note that EVENT_TEXT(10) was moved to EVENT_TEXT(49);
  -- new APPID(10) replaced EVENT_TEXT(10);
  -- new DATA_TYPE's IA(11), WKSTNID(12)
  IF @APPID IS NOT NULL AND @EVENT_TEXT IS NULL
  BEGIN
    -- shift the parameters
    SELECT @EVENT_TEXT = @APPID
    SELECT @APPID = NULL
  END

  -- Make sure EVENT_TEXT is not null
  IF @EVENT_TEXT IS NULL
  BEGIN
    RAISERROR('Process_SA390_Exeption: Null EVENT_TEXT in event with SEQ_NO %s. Ignoring event', 1, 1, @SEQ_NO)
    IF @tcount = 0
      COMMIT TRANSACTION Process_SA390_Exception
    RETURN 0
  END

  EXEC asisp_convertMVSDateTime @DATE_TIME, @datetime OUTPUT

  DECLARE @cid ClassID, @id ObjID
  EXEC @r = asisp_discoverSA390Resource @SourceCID, @SourceID, @FormatType, @ActionType, @SEQ_NO, @SMFID, @RESOURCE_TYPE, @OBJECT_NAME, @EVENT_TYPE, @datetime,
    @cid=@cid OUTPUT, @id=@id OUTPUT
  IF @r < 0 OR @cid IS NULL
  BEGIN
    PRINT 'Process_SA390_Exception: No resource found for SEQ_NO "' + @SEQ_NO + '", SMFID "' + @SMFID + '", RESOURCE_TYPE "' + @RESOURCE_TYPE + '" OBJECT_NAME "' + @OBJECT_NAME + '"'
    GOTO EndProc
  END

  -- Do not go any further if the object is an MVSAddressSpace
  IF @cid = 'MADR'
  BEGIN
    SET @r = 0
    GOTO EndProc
  END

  -- Generate native key
  DECLARE @native_key CHAR(10)
  EXEC asisp_idtonativekey @cid, @id, @native_key OUTPUT

  -- Generate exception on object
  EXEC @r = asisp_createEXCP @EVENT_TYPE, @cid, @id,
    @TIME = @datetime,
    @SEQ_NO = @SEQ_NO,
    @OBJ_TYPE_ID = @cid,
    @OBJECT_NAME = @OBJECT_NAME,
    @NATIVE_KEY = @native_key,
    @EXCP_NAME = @EVENT_TYPE,
    @EXCP_CD = @EVENT_TEXT
  IF @r < 0
  BEGIN
    ROLLBACK TRANSACTION Process_SA390_Exception
    RETURN @r
  END

EndProc:
  -- Commit if we're the outer transaction
  IF @tcount = 0
    COMMIT TRANSACTION Process_SA390_Exception

  RETURN @r
GO
