-- $Header:   //ARCHIVE/Archive/DB/sql/Process_SA390_StateChange.sqv   1.0   Mar 01 2001 20:00:18   BRENT  $
--
-- Process_SA390_StateChange.sql
--

USE Object
GO


---------------------------------------------------------------------------------------------------
-- Process_SA390_StateChange
PRINT 'Creating procedure Process_SA390_StateChange'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'Process_SA390_StateChange' AND type = 'P')
  DROP PROCEDURE Process_SA390_StateChange
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/Process_SA390_StateChange.sqv  $
-- $Revision:   1.0  $
-- $Date:   Mar 01 2001 20:00:18  $
-- $Author:   BRENT  $
--
-- Creates a state change message on a resource managed by SA/390.
--
-- @RESOURCE_TYPE is mapped to a TM390 object class id using the SA390ResourceTypeMap
-- table. If a mapping cannot be established then the resource is considered to be a
-- started task (cid=STC).
--
-- @EVENT_TYPE is the actual state assigned by SA390 (e.g. - UP, STARTING, AUTOTERM).
--
-- @EVENT_TEXT will be NULL for SA390 state changes, so the event text for the 
-- message will be based on @RESOURCE_TYPE, @OBJECT_NAME, and @EVENT_TYPE. The 
-- text is assembled using system message 50701, which in English defaults to 
--    "@RESOURCE_TYPE @OBJECT_NAME IS @EVENT_TYPE"
--
CREATE PROCEDURE Process_SA390_StateChange
  @SourceCID          ClassID,
  @SourceID           ObjID,
  @FormatType         SMALLINT,
  @ActionType         SMALLINT,
  @DATE_TIME          VARCHAR(30) = NULL,
  @SEQ_NO             VARCHAR(50) = NULL,
  @SMFID              VARCHAR(8)  = NULL,
  @RESOURCE_TYPE      VARCHAR(30) = NULL,
  @OBJECT_NAME        VARCHAR(54) = NULL,
  @EVENT_TYPE         VARCHAR(40) = NULL,
  @EVENT_STATE        VARCHAR(40) = NULL,
  @APPID              VARCHAR(255) = NULL,   --- VARCHAR(16) for new Pump shipped in MRD 2000 (TBSM 1.1)
  @WKSTNID            VARCHAR(8)  = NULL,
  @IA                 VARCHAR(16) = NULL,
  @EVENT_TEXT         VARCHAR(255)= NULL
AS
  DECLARE @r INT, @datetime DATETIME

--  RAISERROR('Process_SA390_StateChange @SourceCID=%s, @SourceID=%d, @FormatType=%d, @ActionType=%d', 0, 1,
--            @SourceCID, @SourceID, @FormatType, @ActionType)
  INSERT SA390_StateChange_Audit(SourceCID, SourceID, FormatType, ActionType, DATE_TIME, SEQ_NO, SMFID, RESOURCE_TYPE, OBJECT_NAME, EVENT_TYPE, EVENT_TEXT)
    SELECT @SourceCID, @SourceID, @FormatType, @ActionType, @DATE_TIME, @SEQ_NO, @SMFID, @RESOURCE_TYPE, @OBJECT_NAME, @EVENT_TYPE, @EVENT_TEXT

  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION Process_SA390_StateChange
  ELSE
    SAVE TRANSACTION Process_SA390_StateChange

  -- Special case a literal 'NULL' value being passed into the parameters
  -- We interpret this like a 'real' null.
  IF @EVENT_TEXT = 'NULL' SELECT @EVENT_TEXT = NULL
  IF @EVENT_STATE = 'NULL' SELECT @EVENT_STATE = NULL

  -- Check if receiving data from a pre-MRD 2000 (TBSM 1.1) pump
  --
  -- Note that EVENT_TEXT(10) was moved to EVENT_TEXT(49);
  -- new APPID(10) replaced EVENT_TEXT(10);
  -- new DATA_TYPE's IA(11), WKSTNID(12)
  IF @APPID IS NOT NULL AND @EVENT_TEXT IS NULL
  BEGIN
    -- shift the parameters
    SELECT @EVENT_TEXT = @APPID
    SELECT @APPID = NULL
  END

  EXEC asisp_convertMVSDateTime @DATE_TIME, @datetime OUTPUT

  DECLARE @cid ClassID, @id ObjID
  EXEC @r = asisp_discoverSA390Resource @SourceCID, @SourceID, @FormatType, @ActionType, @SEQ_NO, @SMFID, @RESOURCE_TYPE, @OBJECT_NAME, @EVENT_TYPE, @datetime,
    @cid=@cid OUTPUT, @id=@id OUTPUT
  IF @r < 0 OR @cid IS NULL
  BEGIN
    PRINT 'Process_SA390_StateChange: No resource found for SEQ_NO "' + @SEQ_NO + '", SMFID "' + @SMFID + '", RESOURCE_TYPE "' + @RESOURCE_TYPE + '" OBJECT_NAME "' + @OBJECT_NAME + '"'
    GOTO EndProc
  END

  -- Do not go any further if the object is an MVSAddressSpace
  IF @cid = 'MADR'
  BEGIN
    SET @r = 0
    GOTO EndProc
  END

  -- Dynamically assemble event text
  IF @EVENT_TEXT IS NULL
  BEGIN
    -- In English: "@RESOURCE_TYPE @OBJECT_NAME IS @EVENT_TYPE"
    SELECT @EVENT_TEXT = FORMATMESSAGE(50701, @RESOURCE_TYPE, @OBJECT_NAME, @EVENT_TYPE)
  END

  -- Generate native key
  DECLARE @native_key CHAR(10)
  EXEC asisp_idtonativekey @cid, @id, @native_key OUTPUT

  -- Generate message on object
  DECLARE @MESSAGE VARCHAR(40)
  SELECT @MESSAGE = 'AOF571I_' + @EVENT_TYPE

  PRINT 'Creating message ' + @MESSAGE + ' ON ' + @cid + ' ' + CAST(@id AS VARCHAR(9))
  EXEC @r = asisp_createMESG @MESSAGE, @cid, @id,
    @TIME = @datetime,
    @SEQ_NO = @SEQ_NO,
    @OBJ_TYPE_ID = @cid,
    @OBJECT_NAME = @OBJECT_NAME,
    @NATIVE_KEY = @native_key,
    @STATE = @EVENT_TYPE,
    @MESSAGE = @MESSAGE,
    @REASON_CD = @EVENT_TEXT
  IF @r < 0
  BEGIN
    ROLLBACK TRANSACTION Process_SA390_StateChange
    RETURN @r
  END

EndProc:
  -- Commit if we're the outer transaction
  IF @tcount = 0
    COMMIT TRANSACTION Process_SA390_StateChange

  RETURN @r
GO
