-- $Header:   //ARCHIVE/Archive/DB/sql/_CancelNote.sqv   1.0   Apr 06 2001 05:46:56   BRENT  $
--
-- _CancelNote.sql
--
-- Module Description
--   Defines the _CancelNote stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _CancelNote
PRINT 'Creating procedure _CancelNote'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_CancelNote' AND type = 'P')
  DROP PROCEDURE _CancelNote
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/_CancelNote.sqv  $
-- $Revision:   1.0  $
-- $Date:   Apr 06 2001 05:46:56  $
-- $Author:   BRENT  $
-- Cancels a note that was previously opened with _OpenNote, and may have had events associated
-- with it using _TakeOwnership.
CREATE PROCEDURE _CancelNote
  @cid              ClassID,
  @id               ObjID,
  @noteid           ObjID
AS
  RAISERROR('_CancelNote %s, %d, @noteid=%d', 0, 1, @cid, @id, @noteid)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION CancelNote
  ELSE
    SAVE TRANSACTION CancelNote

  -- If the note has already been committed then we can't cancel it
  DECLARE @commit_time NVARCHAR(40)
  SELECT @commit_time = CONVERT(NVARCHAR(30), ctime, 101) + ' ' + CONVERT(NVARCHAR(8), ctime, 101)
  FROM Note_C WHERE id = @noteid
  IF @@ROWCOUNT <> 0
  BEGIN
    RAISERROR('Note was already been committed at %s', 11, 1, @commit_time)
    ROLLBACK TRANSACTION CancelNote
    RETURN -30000
  END

  -- Delete any NoteDependent and OwnedEvent records for this note
  DELETE NoteDependent WHERE noteid = @noteid
  DELETE OwnedEvent WHERE noteid = @noteid
  DELETE OwnedEventInProgress WHERE noteid = @noteid
      
  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRANSACTION CancelNote
GO
