-- $Header:   //ARCHIVE/Archive/DB/sql/_CommitNote.sqv   1.0   Apr 06 2001 05:47:04   BRENT  $
--
-- _CommitNote.sql
--
-- Module Description
--   Defines the _CommitNote stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _CommitNote
PRINT 'Creating procedure _CommitNote'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_CommitNote' AND type = 'P')
  DROP PROCEDURE _CommitNote
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/_CommitNote.sqv  $
-- $Revision:   1.0  $
-- $Date:   Apr 06 2001 05:47:04  $
-- $Author:   BRENT  $
-- Called as last step of note creation to actually create the note record and to set up the
-- NoteDependent records for the events owned by this note.
CREATE PROCEDURE _CommitNote
  @cid              ClassID,
  @id               ObjID,
  @noteid           ObjID,
  @UserID           VARCHAR(255),
  @Subject          VARCHAR(255),
  @Body             VARCHAR(255),
  @ProblemID        VARCHAR(255) = "",
  @DurationMinutes  INT = 0
AS
  RAISERROR('_CommitNote %s, %d, @noteid=%d, @UserID=''%s'', @Subject=''%s'', @Body=''%s'', @ProblemID=''%s'', @DurationMinutes=%d',
            0, 1, @cid, @id, @noteid, @UserID, @Subject, @Body, @ProblemID, @DurationMinutes)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION CommitNote
  ELSE
    SAVE TRANSACTION CommitNote

  DECLARE @sql VARCHAR(255), @id_str VARCHAR(9)
  SELECT @id_str = CONVERT(VARCHAR(9), @id)

  DECLARE @Status VARCHAR(16), @Resolution VARCHAR(255)
  SELECT @Status = 'Open', @Resolution = ''

  -- Insert records from OwnedEventInProgress into OwnedEvent for this note.
  INSERT INTO OwnedEvent(eventcid, eventid, parentcid, parentid, noteid)
    SELECT DISTINCT eventcid, eventid, parentcid, parentid, noteid
    FROM OwnedEventInProgress AS oep
    WHERE
      noteid = @noteid
      AND NOT EXISTS
      (
        SELECT *
        FROM OwnedEvent AS oe
        WHERE oe.eventcid = oep.eventcid AND oe.eventid = oep.eventid
      )
  IF @@ERROR <> 0
  BEGIN
    ROLLBACK TRANSACTION CommitNote
    EXEC _CancelNote @cid, @id, @noteid
    RETURN -30000
  END

  -- Remove OwnedEventInProgress records for this note
  DELETE FROM OwnedEventInProgress WHERE noteid = @noteid

  -- If this note has no owned events, we assume the calling object to be in the pathological state
  -- of having an alert with no causing events, possibly also having generated one or more child 
  -- events that need to be cleared.
  IF NOT EXISTS (SELECT * FROM OwnedEvent WHERE noteid = @noteid)
  BEGIN
    SET @Status = 'SystemClosed'
    RAISERROR('No OwnedEvents FOR note %d, creating note with %s status', 3, 1, @noteid, @Status)
    DECLARE @dstr VARCHAR(40)
    SELECT @dstr = CONVERT(VARCHAR(40), GETDATE(), 120)
    SELECT @Resolution = FORMATMESSAGE(50900, @dstr)
/* BEGIN OLD CODE
    RAISERROR('No OwnedEvents FOR note %d, creating note anyway', 3, 1, @noteid)
--    ROLLBACK TRANSACTION CommitNote
--    RETURN -30000
    EXEC asisp_setalertstate @cid, @id, 1, '', 0
    SELECT @sql = 'UPDATE ' + cname + '_A SET _HasOwnershipNote=1 WHERE id=' + @id_str + ' ' +
                  'EXEC asisp_notifyHasOwnershipNote 9,' + CONVERT(VARCHAR(9), cno) + ',' + @id_str + ', 1'
      FROM obj_class WHERE cid = @cid
    EXEC(@sql)
    EXEC asisp_negatechildevents @cid, @id 
END OLD CODE */
  END

  -- Insert class record
  INSERT Note_C (id, name) SELECT @noteid, ''
  IF @@ROWCOUNT <> 1
  BEGIN
    ROLLBACK TRANSACTION CommitNote
    EXEC _CancelNote @cid, @id, @noteid
    RETURN -30000
  END
  DECLARE @ctime DATETIME
  SELECT @ctime = ctime FROM Note_C WHERE id = @noteid

  -- If the specified duration is greater than 0, then we schedule a task to check note status at the appropriate time
  DECLARE @expiration DATETIME
  IF @DurationMinutes > 0
    EXEC asisp_schedulenoteexpiration @noteid, @ctime, @DurationMinutes, @UserID
  DECLARE @ExpirationTime VARCHAR(40)
  IF @expiration IS NOT NULL
    SELECT @ExpirationTime = CONVERT(VARCHAR(30), @expiration, 101) + ' ' + CONVERT(VARCHAR(8), @expiration, 108)
  ELSE
    SELECT @ExpirationTime = ''

  -- Insert attribute vector
  SELECT @id_str = CONVERT(VARCHAR(9), @noteid)
  INSERT Note_A
  (
    id,
    _UserID,
    _Type,
    _Status,
    _AlertState,
    _Subject,
    _Body,
    _Resolution,
    _ProblemID,
    _CreatorObjID,
    _CreatorCID,
    _DurationMinutes,
    _ExpirationTime
  )
  SELECT
    @noteid,
    @UserID,
    'Ownership',
    @Status,
    '',
    @Subject,
    @Body,
    @Resolution,
    @ProblemID,
    @id,
    @cid,
    @DurationMinutes,
    @ExpirationTime
  IF @@ROWCOUNT <> 1
  BEGIN
    ROLLBACK TRANSACTION CommitNote
    EXEC _CancelNote @cid, @id, @noteid
    RETURN -30000
  END

  -- If this is an lob, then we need to set the parent cid/id variables to the linked object's
  -- Either way, '@parentcid & @parentid', will contain the physical object's cid/id
  DECLARE @eventcid ClassID, @eventid ObjID, @parentcid ClassID, @parentid ObjID
  SELECT @parentcid = @cid, @parentid = @id
  IF RTRIM(@cid) = 'LOB'
    -- When calling asisp_followphysicallink @parentcid must be non-null if the return parameters are to be filled in
    EXEC asisp_followphysicallink @id, @parentcid output, @parentid output

  -- Insert parent link
  DECLARE @r INT
  EXEC @r = asisp_insertparentlink @parentcid, @parentid, 'NOTE', @noteid
  IF @r < 0
  BEGIN
    ROLLBACK TRANSACTION CommitNote
    EXEC _CancelNote @cid, @id, @noteid
    RETURN @r
  END

  -- Iterate through OwnedEvents for this note and generate the appropriate negating events
  DECLARE @ctime_str VARCHAR(40)
  SELECT @ctime_str = CONVERT(VARCHAR(30), @ctime, 101) + ' ' + CONVERT(VARCHAR(8), @ctime, 108)
  DECLARE OwnedEvent_C INSENSITIVE CURSOR FOR SELECT eventcid, eventid, parentcid, parentid FROM OwnedEvent WHERE noteid = @noteid
  OPEN OwnedEvent_C
  FETCH NEXT FROM OwnedEvent_C INTO @eventcid, @eventid, @parentcid, @parentid
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @sql = 'asisp_negate' + @eventcid + ' ' + CONVERT(VARCHAR(9), @eventid) + ', ''' + @UserID + ''', ''' + @ctime_str + ''''
    EXEC(@sql)
    IF @@ERROR <> 0
    BEGIN
      ROLLBACK TRANSACTION CommitNote
      DEALLOCATE OwnedEvent_C
      EXEC _CancelNote @cid, @id, @noteid
      RETURN -30000
    END

    -- The object that owns the event becomes a note dependent
    RAISERROR('NoteDependent: %d %s %d %s %d', 0, 1, @noteid, @parentcid, @parentid, @eventcid, @eventid)
    INSERT NoteDependent(noteid, depcid, depid, eventcid, eventid)
      SELECT @noteid, @parentcid, @parentid, @eventcid, @eventid

    -- If the object that owns the event is not also the object creating the note, then the object
    -- creating the note must also explicitly be made a note dependent
    IF @parentcid <> @cid OR @parentid <> @id
      INSERT NoteDependent(noteid, depcid, depid, eventcid, eventid)
      SELECT @noteid, @cid, @id, @eventcid, @eventid

    FETCH NEXT FROM OwnedEvent_C INTO @eventcid, @eventid, @parentcid, @parentid
  END
  DEALLOCATE OwnedEvent_C

  -- Debug statement -- comment out if not needed
--  PRINT 'Note dependents after negating events'
--  EXEC asisp_displaynotedependents @noteid

  -- The physical object creating the note is also a note dependent
  INSERT NoteDependent(noteid, depcid, depid, eventcid, eventid)
    SELECT DISTINCT @noteid, @parentcid, @parentid, n.eventcid, n.eventid
    FROM NoteDependent n
    WHERE 
      n.noteid = @noteid 
      AND NOT EXISTS 
      (
        SELECT * FROM NoteDependent nn 
        WHERE nn.noteid=@noteid AND nn.depcid=@parentcid AND nn.depid=@parentid AND nn.eventcid=n.eventcid AND nn.eventid=n.eventid
      )
  IF @@ERROR <> 0
  BEGIN
    RAISERROR('_CommitNote aborting due to error inserting creating object as note dependent on note %d', 11, 1, @noteid)
    ROLLBACK TRANSACTION CommitNote
    EXEC _CancelNote @cid, @id, @noteid
    RETURN -3000
  END

  -- Debug statement -- comment out IF NOT needed
--  PRINT 'Note dependents after adding creating object'
--  EXEC asisp_displaynotedependents @noteid

  -- Any object that is linked to one of the objects getting an owned event is also a note dependent
  INSERT NoteDependent(noteid, depcid, depid, eventcid, eventid)
    SELECT DISTINCT @noteid, l.src_cid, l.src_id, nd.eventcid, nd.eventid
    FROM link AS l 
      JOIN NoteDependent AS nd ON l.dst_cid = nd.depcid AND l.dst_id = nd.depid
    WHERE 
      nd.noteid = @noteid 
      AND l.link_type = 'LOBL' 
      AND l.deleted = 0
      AND NOT EXISTS 
      (
        SELECT * FROM NoteDependent AS ndprev 
        WHERE
          ndprev.noteid = @noteid
          AND ndprev.depcid = l.src_cid
          AND ndprev.depid = l.src_id
          AND ndprev.eventcid = nd.eventcid
          AND ndprev.eventid = nd.eventid
      )
  IF @@ERROR <> 0
  BEGIN
    RAISERROR('_CommitNote aborting due to error inserting linked objects for note dependents on note %d', 11, 1, @noteid)
    ROLLBACK TRANSACTION CommitNote
    EXEC _CancelNote @cid, @id, @noteid
    RETURN -3000
  END

  -- Debug statement -- comment out IF NOT needed
--  PRINT 'Note dependents after adding linked objects'
--  EXEC asisp_displaynotedependents @noteid

  -- Iterate through the note dependents and set the hasownershipnote flag on them
  DECLARE notify_obj_c INSENSITIVE CURSOR FOR 
    SELECT DISTINCT depcid, CONVERT(NVARCHAR(9),depid) FROM NoteDependent WHERE noteid = @noteid
  OPEN notify_obj_c
  DECLARE @notify_cid ClassID, @notify_id_str VARCHAR(9)
  FETCH NEXT FROM notify_obj_c INTO @notify_cid, @notify_id_str
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    -- UPDATE the object's HasOwnershipNote status
    SELECT @sql = 'UPDATE ' + cname + '_A SET _HasOwnershipNote = 1 WHERE id = ' + @notify_id_str +
                  ' EXEC asisp_notifyHasOwnershipNote 9,' + CONVERT(NVARCHAR(9), cno) + ',' + @notify_id_str + ',1'
    FROM obj_class
    WHERE cid = @notify_cid
--    PRINT @sql
    EXEC(@sql)
    FETCH NEXT FROM notify_obj_c INTO @notify_cid, @notify_id_str
  END
  DEALLOCATE notify_obj_c

  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRANSACTION CommitNote
GO
