-- $Header:   //ARCHIVE/Archive/DB/sql/_PollForCompletedSchedules.sqv   1.0   Mar 17 2001 12:11:04   BRENT  $
--
-- _PollForCompletedSchedules.sql
--

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _PollForCompletedSchedules
PRINT 'Creating procedure _PollForCompletedSchedules'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_PollForCompletedSchedules' AND type = 'P')
  DROP PROCEDURE _PollForCompletedSchedules
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/_PollForCompletedSchedules.sqv  $
-- $Revision:   1.0  $
-- $Date:   Mar 17 2001 12:11:04  $
-- $Author:   BRENT  $
-- Checks for batch schedules that contain only completed jobs that are not themselves
-- in a completed state. Generates a 'OPCENDED' message on such schedules and directly
-- sets their state to 'Completed'.
-- This procedure is installed as a scheduled task that runs periodically throughout
-- the day.
CREATE PROCEDURE _PollForCompletedSchedules
  @cid     ClassID,
  @id      ObjID,
  @verbose TINYINT = 0
AS
  SET NOCOUNT ON

  -- StateType 83 is 'Completed' for BATC
  -- StateType 93 is 'Completed' for BCYS

  -- Build temp table that lists the distinct states of jobs within schedules that are not
  -- in a Completed state
  SELECT DISTINCT s.id AS id, s._CurrentStateID AS sched_state, b._CurrentStateID AS job_state
  INTO #btcy_job_state
  FROM BATCH_V AS b JOIN BatchCycle_V AS s ON s.id = b._ParentID
  WHERE b._ParentCID = 'BTCY' AND s._CurrentStateID <> 93

  -- Delete from the temp table records for any schedules that have jobs that are not in a Completed state
  DELETE b1 FROM 
    #btcy_job_state AS b1
    JOIN #btcy_job_state AS b2 ON b2.id = b1.id
  WHERE 
    ((b1.job_state = 83 AND b2.job_state <> 83) OR b1.job_state <> 83)

  -- If there are any remaining records these indicate schedules that are not in a Completed state
  -- which have all jobs in a completed state. We generate a message on each such schedule to indicate
  -- it too should be in a Completed state.
  DECLARE @nMesgs INT
  SELECT @nMesgs = COUNT(*) FROM #btcy_job_state
  IF @nMesgs > 0
  BEGIN
    PRINT '_PollForCompletedSchedules: Generating ' + CAST(@nMesgs AS VARCHAR) + ' completed message(s) for schedules'

    INSERT StagedMESG(parent_cid, parent_id, TIME, SEQ_NO, OBJECT_NAME, NATIVE_KEY, DISCOVERY_IND, STATE, MESSAGE, REASON_CD)
      SELECT 'BTCY', b.id, GETDATE(), '', s.name, '', 'MON', 'COMPLETED', 'OPCENDED', 'ALL OPERATIONS IN APPLICATION ' + s.name + ' COMPLETED AS OF ' + CONVERT(VARCHAR(40), GETDATE(), 120)
      FROM #btcy_job_state AS b JOIN BatchCycle_V AS s ON s.id = b.id
    UPDATE btcy SET _CurrentStateID = 93 FROM BatchCycle_V AS btcy JOIN #btcy_job_state AS js ON js.id = btcy.id
  END
  ELSE
    PRINT '_PollForCompletedSchedules: no messages generated'
GO
