-- $Header:   //ARCHIVE/Archive/DB/sql/_TakeOwnership.sqv   1.0   Apr 06 2001 05:47:18   BRENT  $
--
-- _TakeOwnership.sql
--
-- Module Description
--   Defines the _TakeOwnership stored procedure

USE Object
GO


---------------------------------------------------------------------------------------------------
-- _TakeOwnership
PRINT 'Creating procedure _TakeOwnership'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_TakeOwnership' AND type = 'P')
  DROP PROCEDURE _TakeOwnership
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/_TakeOwnership.sqv  $
-- $Revision:   1.0  $
-- $Date:   Apr 06 2001 05:47:18  $
-- $Author:   BRENT  $
-- Takes ownership of a specified event or all events on a the calling object.
-- This is done by associating the specified @noteid to the events in the OwnedEvent table.
CREATE PROCEDURE _TakeOwnership
  @cid        ClassID,
  @id         ObjID,
  @noteid     ObjID,
  @eventcid   ClassID = NULL,
  @eventid    ObjID   = 0,
  @top_level  INT     = 1
AS
  RAISERROR('_TakeOwnership ''%s'', %d, @noteid=%d, @eventcid=%s, @eventid=%d, @top_level=%d', 0, 1,
            @cid, @id, @noteid, @eventcid, @eventid, @top_level)

  IF @top_level = 1
    CREATE TABLE #descendents ( cid ClassID, id ObjID, gen INT )

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION _TakeOwnership
  ELSE
    SAVE TRANSACTION _TakeOwnership

  -- If this is an LOB, then we need to set the parent cid/id variables to the linked object's
  DECLARE @parentcid ClassID, @parentid ObjID
  SELECT @parentcid = @cid, @parentid = @id
  IF RTRIM(@cid) = 'LOB'
    -- When calling asisp_followphysicallink @parentcid must be non-null if the return parameters are to be filled in
    EXEC asisp_followphysicallink @id, @parentcid output, @parentid output

  IF @eventcid = 'EXCP'
  BEGIN
    INSERT INTO OwnedEventInProgress(eventcid, eventid, parentcid, parentid, noteid)
      SELECT @eventcid, @eventid, _ParentCID, _ParentID, @noteid
      FROM Exception_V WHERE id = @eventid
  END
  ELSE IF @eventcid = 'MESG'
  BEGIN
    INSERT INTO OwnedEventInProgress(eventcid, eventid, parentcid, parentid, noteid)
      SELECT @eventcid, @eventid, _ParentCID, _ParentID, @noteid
      FROM Message_V WHERE id = @eventid
  END
  ELSE IF @eventcid IS NULL
  BEGIN -- take ownership of everything wrong on the object
    -- Get the last non-ignore message on the object.  If its alert state is not green then take ownership of it.
    DECLARE @AlertState INT
    SELECT @AlertState=AlertState, @eventid=mesg_id
      FROM CurrentMessageAlertStatePri 
      WHERE cid=@parentcid AND id=@parentid
    IF @@ROWCOUNT=1 AND @AlertState > 1
    BEGIN
      PRINT 'Inserting OwnedEventInProgress record for last non-ignore message non-green message ' + CAST(@eventid AS VARCHAR(9))
      INSERT INTO OwnedEventInProgress(eventcid, eventid, parentcid, parentid, noteid) 
        SELECT 'MESG', @eventid, @parentcid, @parentid, @noteid
    END
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('Could not insert OwnedEventInProgress for eventcid=MESG, eventid=%d, parentcid=%s, parentid=%d, noteid=%d', 11, 1,
                @eventid, @parentcid, @parentid, @noteid)
      ROLLBACK TRANSACTION _TakeOwnership
      RETURN -30000
    END

    -- Take ownership of all exceptions
    PRINT 'Inserting OwnedEventInProgress records for all exceptions on ' + @cid + ' ' + CAST(@id AS VARCHAR(9))
    INSERT INTO OwnedEventInProgress(eventcid, eventid, parentcid, parentid, noteid) 
      SELECT 'EXCP', id, @parentcid, @parentid, @noteid
      FROM Exception_V AS e WHERE _ParentCID=@parentcid AND _ParentID=@parentid AND deleted = 0
    IF @@ERROR <> 0
    BEGIN
      RAISERROR('Could not insert OwnedEventInProgress for eventcid=EXCP, parentcid=%s, parentid=%d, noteid=%d', 11, 1,
                @parentcid, @parentid, @noteid)
      ROLLBACK TRANSACTION _TakeOwnership
      RETURN -30000
    END
    IF @top_level = 1 -- Build list of descendents that have sent child events to this object. Note that in the case of
                      -- LOBs the descendents are the LOB descendents, *NOT* the linked object's
    BEGIN
      PRINT '_TakeOwnership: building list of descendents'
      DECLARE @gen INT
      SELECT @gen = 1
      INSERT #descendents 
        SELECT DISTINCT _ChildCID, _ChildID, @gen
        FROM ChildEvent_V
        WHERE _ParentCID = @cid AND _ParentID = @id AND deleted = 0
      WHILE @@ROWCOUNT > 0
      BEGIN
        RAISERROR('_TakeOwnership: generation=%d', 0, 1, @gen)
        SELECT @gen = @gen + 1
        INSERT #descendents 
          SELECT DISTINCT _ChildCID, _ChildID, @gen
          FROM ChildEvent_V ce, #descendents d
          WHERE _ParentCID = d.cid AND _ParentID = d.id AND deleted = 0 AND gen = @gen - 1
      END
      DECLARE TakeOwnership_c CURSOR FOR SELECT cid, id FROM #descendents

      OPEN TakeOwnership_c
      DECLARE @childcid ClassID, @childid ObjID
      FETCH NEXT FROM TakeOwnership_c INTO @childcid, @childid
      WHILE @@FETCH_STATUS <> -1
      BEGIN
        -- Take ownership on descendent, specifying @top_level to be 0 so that this loop will not be repeated
        EXEC _TakeOwnership @childcid, @childid, @noteid, @top_level=0
        FETCH NEXT FROM TakeOwnership_c INTO @childcid, @childid
      END
      DEALLOCATE TakeOwnership_c
    END
  END
  ELSE
  BEGIN
    RAISERROR('_TakeOwnership: @eventcid must be NULL, ''EXCP'', OR ''MESG''', 11, 1)
    ROLLBACK TRANSACTION _TakeOwnership
    IF @top_level = 1
      DROP TABLE #descendents
    RETURN -30000
  END
  IF @tcount = 0
    COMMIT TRANSACTION _TakeOwnership
  IF @top_level = 1
    DROP TABLE #descendents
  RAISERROR('TakeOwnership finished successfully: @@nestlevel=%d', 0, 1, @@nestlevel)
GO
