-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/asisp_checkForDuplicateProcess.sqv   1.3   May 17 2001 13:29:12   BRENT  $
--
-- asisp_checkForDuplicateProcess.sql
--

---------------------------------------------------------------------------------------------------
-- asisp_checkForDuplicateProcess
PRINT 'Creating procedure asisp_checkForDuplicateProcess'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_checkForDuplicateProcess' AND type = 'P')
  DROP PROCEDURE asisp_checkForDuplicateProcess
GO

-- Checks to see if there is another system process with the same name as the current process.
-- Selects the program_name, hostname, and hostprocess columns from the sysprocesses table if
-- there are such processes.
-- Returns 1 if there is a duplicate, or 0 if the current program_name is unique.
CREATE PROCEDURE asisp_checkForDuplicateProcess
  @OldSPID  INT=NULL -- If used then the specified process id is not considered a duplicate.
                     -- This is useful in the case of deadlock recovery where the old
                     -- process may still be hanging around.
AS
  -- Lookup our program_name
  DECLARE @program_name NVARCHAR(255)
  SELECT @program_name = RTRIM(program_name) FROM sysprocesses WHERE spid = @@SPID

  -- See if there is another connection with this program name
  SELECT DISTINCT
    RTRIM(CAST(program_name AS VARCHAR(255))) AS program_name,
    RTRIM(CAST(hostname AS VARCHAR(255))) AS hostname,
    hostprocess,
    spid
  INTO #dupspid
  FROM
    sysprocesses
  WHERE
    program_name = @program_name
    AND spid <> @@SPID
  IF @@ROWCOUNT = 0
    RETURN 0

  DELETE FROM #dupspid WHERE spid = @OldSPID
  IF @@ROWCOUNT = 1
    RAISERROR('asisp_checkForDuplicateProcess: Old process [%d] detected for program [%s], ignoring', 2, 1, @OldSPID, @program_name)

  SELECT * FROM #dupspid
  IF @@ROWCOUNT = 0
    RETURN 0
  ELSE
    RETURN 1
GO
