-- $Header:   //ARCHIVE/Archive/DB/SQL/asisp_init_lob_link.sqv   1.0   Mar 07 2001 11:31:46   BRENT  $
--
-- asisp_init_lob_link.sql

USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_init_lob_link
PRINT 'Creating procedure asisp_init_lob_link'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_init_lob_link' AND type = 'P')
  DROP PROCEDURE asisp_init_lob_link
GO

-- $Archive:   //ARCHIVE/Archive/DB/SQL/asisp_init_lob_link.sqv  $
-- $Revision:   1.0  $
-- $Date:   Mar 07 2001 11:31:46  $
-- $Author:   BRENT  $
-- Initializes the lob_link table for the specified LOB. This should be called only for a new LOB
-- since it only initializes lob_link for the ancestors of the specified. Use asisp_sync_lob_link
-- to resynchronize lob_link for both the ancestors and descendents of an existing LOB.
--
-- NOTE: This procedure will not handle *reinitializing* an LOB. Use asisp_sync_lob_link instead
create procedure asisp_init_lob_link
  @lob_id   ObjID,
  @phy_cid  ClassID,
  @phy_id   ObjID
as
  SET NOCOUNT ON
  PRINT 'asisp_init_lob_link @lob_id=' + CAST(@lob_id AS VARCHAR)
    + ', @phy_cid = ' + CASE WHEN @phy_cid IS NULL THEN 'NULL' ELSE '''' + @phy_cid + '''' END
    + ', @phy_id = ' + CASE WHEN @phy_id IS NULL THEN 'NULL' ELSE CAST(@phy_id AS VARCHAR) END
   
  -- Standard transaction logic
  declare @tcount int
  select @tcount = @@trancount
  if @tcount = 0
    begin tran asisp_init_lob_link
  else
    save tran asisp_init_lob_link

  declare @err int, @rc int

  declare @gen int
  select @gen = 1
  -- Get rid of all previous entries where this LOB is on the dst side
  delete lob_link where dst_id = @lob_id
  if @@error <> 0
  begin
    rollback tran asisp_init_lob_link
  end

  -- If this LOB is linked to another LOB lookup the ultimate physical object the specified LOB is linked to
  if @phy_cid = 'LOB'
  begin
    exec asisp_followphysicallink @phy_id, @phy_cid OUT, @phy_id OUT
    if @@error <> 0
    begin
      rollback tran asisp_init_lob_link
    end
    if @phy_cid = 'LOB' select @phy_cid = null, @phy_id = 0
  end
    
  -- Insert the ancestor records for the specified LOB.
  -- If this is a top-level LOB then insert a lob_link entry with this LOB on the dst side
  if exists (select * from link where src_cid = 'LOBC' and src_id = 1 and dst_cid = 'LOB' and dst_id = @lob_id)
  begin
    insert lob_link(src_id, dst_id, gen, phy_cid, phy_id)
      select 0, @lob_id, @gen, @phy_cid, @phy_id
    if @@error <> 0
    begin
      rollback tran asisp_init_lob_link
    end
  end
  else
  begin
--raiserror('asisp_init_lob_link @lob_id=%d, @phy_cid=''%s'', @phy_id=%d: gen = %d', 2, 1, @lob_id, @phy_cid, @phy_id, @gen)
    select @gen = 1
    declare @src_id ObjID
    -- Lookup parent of this LOB
    select @src_id = src_id from LOB_LOB_LOBC where dst_id = @lob_id
    if @@error <> 0
    begin
      rollback tran asisp_init_lob_link
      return -30000
    end
--raiserror('asisp_init_lob_link @lob_id=%d, @phy_cid=''%s'', @phy_id=%d: src_id = %d', 2, 1, @lob_id, @phy_cid, @phy_id, @src_id)
    if @src_id IS NOT null
    begin
      -- Insert record for immediate ancestor of this LOB
      insert lob_link(src_id, dst_id, gen, phy_cid, phy_id)
        select @src_id, @lob_id, @gen, @phy_cid, @phy_id
      if @@error <> 0
      begin
        rollback tran asisp_init_lob_link
        return -30000
      end

      -- Insert records based on the rows already in lob_link that represent ancestors of the specified
      -- LOB's parent. We want exactly these records, but with the specified LOB on the dst side and 
      -- a generation count one greater.
      insert lob_link(src_id, dst_id, gen, phy_cid, phy_id)
        select p.src_id, @lob_id, gen + @gen, @phy_cid, @phy_id
        from lob_link p
        where p.dst_id = @src_id
      if @@error <> 0
      begin
        rollback tran asisp_init_lob_link
        return -30000
      end

      -- If this lob is linked to another lob, then it needs to be treated as shortcut to that lob,
      -- which means that wherever the target lob appears in lob_link on the destination side we 
      -- need an equivalent record where this lob is on the source side.
      insert lob_link
        select ll.src_id, l.dst_id, l.gen, l.phy_cid, l.phy_id
        from LOB_LOB_LOBL as ll join lob_link as l on l.src_id = ll.dst_id
        where ll.src_id = @lob_id
      select @err = @@error, @rc = @@rowcount
      if @err <> 0
      begin
        rollback tran asisp_init_lob_link
        return -30000
      end
      if @rc > 0
      begin
        insert lob_link
          select lsrc.src_id, ldst.dst_id, ldst.gen+lsrc.gen as gen, ldst.phy_cid, ldst.phy_id
          from lob_link as ldst join lob_link as lsrc on lsrc.dst_id = ldst.src_id
          where ldst.src_id = @lob_id
            and not exists (select * from lob_link prev where prev.src_id = lsrc.src_id and prev.dst_id = ldst.dst_id)
        select @err = @@error, @rc = @@rowcount
        if @err <> 0
        begin
          rollback tran asisp_init_lob_link
          return -30000
        end
      end
    end
  end

  if @tcount = 0
    commit tran asisp_init_lob_link
GO
