-- $Header:   //ARCHIVE/Archive/DB/sql/asisp_processStagedSJMs.sqv   1.0   May 14 2001 12:20:26   BRENT  $
--
-- asisp_processStagedSJMs.sql
--
-- Module Description
--   Defines the asisp_processStagedSJMs stored procedure

USE Object
GO

---------------------------------------------------------------------------------------------------
-- asisp_processStagedSJMs
PRINT 'Creating procedure asisp_processStagedSJMs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_processStagedSJMs' AND type = 'P')
  DROP PROCEDURE asisp_processStagedSJMs
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/asisp_processStagedSJMs.sqv  $
-- $Revision:   1.0  $
-- $Date:   May 14 2001 12:20:26  $
-- $Author:   BRENT  $
CREATE PROCEDURE asisp_processStagedSJMs
  @rowlimit int = 1000,
  @debug    tinyint = 0
AS
  -- exit quickly if there is no data to process
  IF NOT EXISTS (SELECT * FROM StagedSJM)
  BEGIN
    RAISERROR('asisp_processStagedSJMs: no staged events to process', 0, 1)
    RETURN 0
  END

  RAISERROR('asisp_processStagedSJMs: rowcount=%d', 0, 1, @rowlimit)
  SET ROWCOUNT @rowlimit

  DECLARE @rc int, @err int, @msg varchar(255), @chkpt int
  SELECT @chkpt = 1

  -- Create temp table with ids of all SJM events that are available for processing, 'available'
  -- being defined as a SJM whose parent object does NOT have any earlier MESG events that have
  -- NOT been processed. Note that not all fields from StagedSJM are included since they are not
  -- currently in use. These are indicated by the comments in the table definition.
  CREATE TABLE #available_sjm 
  (
    timestamp         binary(8) PRIMARY KEY,
    ctime             datetime,
    parent_cid        ClassID,
    parent_id         ObjID,
    DATE_TIME         datetime,
    SEQ_NO            varchar(80) null,
--    OBJECT_NAME       ObjName null,
    CHILD_OBJ_TYPE    ClassID null,
    CHILD_OBJ_NAME    ObjName null,
    MESSAGE           varchar(16) null,
    STATE             varchar(40) null,
    REASON_CD         varchar(255) null,
    DISCOVERY_IND     varchar(4) null, 
    PARM_1            varchar(128) null,
    PARM_2            varchar(128) null,
    PARM_3            varchar(128) null,
    PARM_4            varchar(128) null,
--    PARM_5            varchar(128) null,
--    PARM_6            varchar(128) null,
--    PARM_7            varchar(128) null,

    -- These are fields for holding the values extracted from the REASON_CD field
    field1            varchar(100) null,
    field2            varchar(100) null,
    field3            varchar(100) null,
    field4            varchar(100) null
  )
  INSERT #available_sjm
    SELECT
      timestamp,
      ctime,
      parent_cid,
      parent_id,
      DATE_TIME,
      SEQ_NO,
--      OBJECT_NAME,
      CHILD_OBJ_TYPE,
      CHILD_OBJ_NAME,
      MESSAGE,
      STATE,
      REASON_CD,
      DISCOVERY_IND,
      PARM_1,
      PARM_2,
      PARM_3,
      PARM_4,
      null,
      null,
      null,
      null
    FROM StagedSJM f
    WHERE
      f.timestamp NOT in (SELECT evt_timestamp FROM ProcessedStagedEvent)
      AND NOT EXISTS
      (
        SELECT * FROM StagedMESG m
        WHERE 
          m.parent_cid = f.parent_cid AND m.parent_id = f.parent_id
          AND m.timestamp < f.timestamp
          AND NOT EXISTS (SELECT * FROM ProcessedStagedEvent WHERE evt_timestamp = m.timestamp)
      )
  SELECT @rc = @@rowcount, @err = @@error
  EXEC master..xp_asisprintf @msg output, 'asisp_processStagedSJMs: @@error=%d, @@rowcount=%d', @err, @rc
  EXEC asisp_checkpointmessage @msg, 1, @chkpt
  SELECT @chkpt = @chkpt + 1
  IF @err <> 0
  BEGIN
    RETURN -30000
  END
  IF @rc = 0
  BEGIN
    RETURN 0
  END

  -- Standard transaction logic
  DECLARE @tcount int
  SELECT @tcount = @@trancount
  IF @tcount = 0
    BEGIN TRANSACTION asisp_processStagedSJMs
  ELSE
    SAVE TRANSACTION asisp_processStagedSJMs

  EXEC asisp_checkpointmessage 'asisp_processStagedSJMs: TRAN begin', 0, @chkpt
  IF @debug = 1
    SELECT @chkpt AS chkpt, * FROM #available_sjm
  SELECT @chkpt = @chkpt + 1


  -- Also put event timestamp records into a table we will use to insert ProcessStagedEvent
  -- when successfully finished
  SELECT timestamp into #processed_sjm FROM #available_sjm
  DECLARE @num_events_processed int
  SELECT @num_events_processed = count(*) FROM #processed_sjm
  EXEC asisp_checkpointmessage 'asisp_processStagedSJMs: inserting into StagedFMSG', 0, @chkpt
  SELECT @chkpt = @chkpt + 1

  -- There are two general categories of subjob messages:
  -- 1. Messages with the child object identified explicitly (usually in the CHILD_OBJ_NAME field)
  -- 2. Messages where the child object must be identified by pattern matching on the message text.
  --
  -- Messages of the first category tend to be (re)discovery messages and come in large sets.
  -- These we handle explicitly.

  -- Handle FILE FCT messages  
  INSERT StagedFMSG(timestamp, ctime, parent_cid, parent_id, DATE_TIME, SEQ_NO, MESSAGE, REASON_CD, FileName, BatchNo, FileEnabled, FileOpen, Dataset)
    SELECT
      timestamp,
      ctime,
      parent_cid,
      parent_id,
      DATE_TIME,
      SEQ_NO,
      MESSAGE,
      REASON_CD,
      CONVERT(VARCHAR(8),CHILD_OBJ_NAME) AS FileName,
      CASE WHEN PARM_4 LIKE 'BATCH%' THEN CONVERT(INT, SUBSTRING(PARM_4, 6, LEN(PARM_4)-5)) ELSE 0 END AS BatchNo,
      CONVERT(VARCHAR(3), PARM_1) AS FileEnabled,
      CONVERT(VARCHAR(3), PARM_2) AS FileOpen,
      PARM_3 AS Dataset
    FROM
      #available_sjm
    WHERE
      CHILD_OBJ_TYPE = 'FILE'
      AND PARM_1 IS NOT NULL
  IF @@rowcount > 0
    DELETE #available_sjm
    WHERE
      CHILD_OBJ_TYPE = 'FILE'
      AND PARM_1 IS NOT NULL

  -- Handle TRAN discovery messages
  INSERT StagedTRANDISC(timestamp, ctime, parent_cid, parent_id, DATE_TIME, SEQ_NO, MESSAGE, REASON_CD, TranName, TranStatus, TranPriority, TranClass, ProgramID, TWASize)
    SELECT
      timestamp,
      ctime,
      parent_cid,
      parent_id,
      DATE_TIME,
      SEQ_NO,
      CONVERT(VARCHAR(16),MESSAGE),
      REASON_CD,
      CONVERT(VARCHAR(8),CHILD_OBJ_NAME) AS TranName,
      CONVERT(VARCHAR(16),STATE)  AS TranStatus,
      CONVERT(INT, PARM_1) AS TranPriority,
      CONVERT(VARCHAR(16),PARM_2) AS TranClass,
      CONVERT(VARCHAR(8),PARM_3) AS ProgramID,
      CONVERT(INT, PARM_4) AS TWASize
    FROM
      #available_sjm
    WHERE
      CHILD_OBJ_TYPE = 'TRAN'
      AND PARM_1 IS NOT NULL
  IF @@rowcount > 0
    DELETE #available_sjm
    WHERE
      CHILD_OBJ_TYPE = 'TRAN'
      AND PARM_1 IS NOT NULL

  -- The remaining messages in #available_sjm now have to be pattern-matched in order to identify
  -- the child objects.
  DECLARE @n INT
  SELECT @n = COUNT(*) FROM #available_sjm
  SET @msg = 'asisp_processStagedSJMs: Performing pattern-matching on ' + CAST(@n AS VARCHAR(9)) + ' message(s)'
  EXEC asisp_checkpointmessage @msg, 0, @chkpt
  IF @debug = 1
    SELECT @chkpt AS chkpt, * FROM #available_sjm
  SELECT @chkpt = @chkpt + 1

  -- Create a regular expression context for handling the extraction of data from the message text
  DECLARE @regex int, @hr int
  EXEC @hr = sp_OACreate 'TMRegExp.RegExContext', @regex OUT
  IF @hr <> 0
  BEGIN
    DECLARE @errtext VARCHAR(255)
    EXEC sp_OAGetErrorInfo NULL, NULL, @errtext OUT
    RAISERROR('%s', 3, 1, @errtext)
    ROLLBACK TRANSACTION asisp_processStagedSJMs
    RETURN -30000
  END

  -- For each distinct message, we look up its search expression and processing pattern from SJMessageMap
  DECLARE sjmmc INSENSITIVE CURSOR FOR
    SELECT DISTINCT mm.MsgID, mm.State, mm.SearchExpression, mm.ReplaceExpression
    FROM SJMessageMap mm JOIN #available_sjm AS a on a.MESSAGE = mm.MsgID
  OPEN sjmmc
  DECLARE @MsgID varchar(16), @State varchar(40), @SearchExpression varchar(255), @ReplaceExpression varchar(255)
  FETCH NEXT FROM sjmmc INTO @MsgID, @State, @SearchExpression, @ReplaceExpression
  WHILE @@fetch_status <> -1
  BEGIN
    -- Compile the regular expression into the processing context

    EXEC @hr = sp_OAMethod @regex, 'Compile', NULL, @SearchExpression, 0
    IF @hr <> 0
    BEGIN
      EXEC sp_OAGetErrorInfo @regex, NULL, @errtext OUT
      RAISERROR('%s', 3, 1, @errtext)
      DEALLOCATE sjmmc
      ROLLBACK TRANSACTION asisp_processStagedSJMs
      RETURN -30000
    END

    -- Now cursor through each message of this type and perform the processing
    DECLARE sjmc CURSOR FOR SELECT timestamp, REASON_CD FROM #available_sjm WHERE MESSAGE = @MsgID
    OPEN sjmc
    DECLARE @timestamp binary(8), @REASON_CD varchar(255), @update_sql varchar(255)
    FETCH NEXT FROM sjmc INTO @timestamp, @REASON_CD
    WHILE @@fetch_status <> -1
    BEGIN
      IF @debug = 1
      BEGIN
        PRINT 'Searching for pattern in text'
        PRINT '  PATTERN: ' + @SearchExpression
        PRINT '  TEXT:    ' + @REASON_CD
      END
      -- Search the text
      EXEC @hr = sp_OAMethod @regex, 'Exec', NULL, @REASON_CD, 0
      IF @hr <> 0
      BEGIN
        EXEC sp_OAGetErrorInfo @regex, NULL, @errtext OUT
        RAISERROR('PATTERN=[%s]', 3, 1, @SearchExpression)
        RAISERROR('TEXT=[%s]', 3, 1, @REASON_CD)
        RAISERROR('ERROR=[%s]', 3, 1, @errtext)
        DEALLOCATE sjmc
        DEALLOCATE sjmmc
        ROLLBACK TRANSACTION asisp_processStagedSJMs
        RETURN -30000
      END
      -- Replace the matched values in the processing pattern
      DECLARE @processing_sql varchar(255)
      EXEC @hr = sp_OAMethod @regex, 'Replace', @update_sql OUT, @ReplaceExpression
      IF @hr <> 0
      BEGIN
        EXEC sp_OAGetErrorInfo @regex, NULL, @errtext OUT
        RAISERROR('%s', 3, 1, @errtext)
        DEALLOCATE sjmc
        DEALLOCATE sjmmc
        ROLLBACK TRANSACTION asisp_processStagedSJMs
        RETURN -30000
      END
      IF @debug = 1
      BEGIN
        PRINT '  STATE: ' + @State
        PRINT '  UPDATE_SQL: ' + @update_sql
      END
      -- Execute the processed SQL expression
      EXEC('UPDATE #available_sjm SET STATE=''' + @State + ''',' + @update_sql + ' WHERE CURRENT OF sjmc')
      IF @@error <> 0
      BEGIN
        DEALLOCATE sjmc
        DEALLOCATE sjmmc
        ROLLBACK TRANSACTION asisp_processStagedSJMs
        RETURN -30000
      END
      FETCH NEXT FROM sjmc INTO @timestamp, @REASON_CD
    END
    DEALLOCATE sjmc

    FETCH NEXT FROM sjmmc INTO @MsgID, @State, @SearchExpression, @ReplaceExpression
  END
  DEALLOCATE sjmmc

  SET @msg = 'asisp_processStagedSJMs: finished pattern-matching, inserting into StagedFILEMESG'
  EXEC asisp_checkpointmessage @msg, 0, @chkpt
  IF @debug = 1
    SELECT @chkpt AS chkpt, * FROM #available_sjm
  SELECT @chkpt = @chkpt + 1

  -- Insert into FILE and TRAN specific staging tables for further processing
  INSERT StagedFILEMESG 
  (
    timestamp,
    ctime,
    parent_cid,
    parent_id,
    DATE_TIME,
    SEQ_NO,
    MESSAGE,
    STATE,
    REASON_CD,
    DISCOVERY_IND,
    FileName,
    Dataset
  )
  SELECT
    timestamp,
    ctime,
    parent_cid,
    parent_id,
    DATE_TIME,
    SEQ_NO,
    CONVERT(VARCHAR(16),MESSAGE),
    CONVERT(VARCHAR(40),STATE),
    REASON_CD,
    CONVERT(VARCHAR(4),DISCOVERY_IND),
    CONVERT(VARCHAR(8),field1),
    field2
  FROM #available_sjm
  WHERE CHILD_OBJ_TYPE = 'FILE'

  SET @msg = 'asisp_processStagedSJMs: INSERT INTO StagedTRANMESG'
  EXEC asisp_checkpointmessage @msg, 0, @chkpt
  IF @debug = 1
    SELECT @chkpt AS chkpt, * FROM #available_sjm
  SELECT @chkpt = @chkpt + 1

  INSERT StagedTRANMESG 
  (
    timestamp,
    ctime,
    parent_cid,
    parent_id,
    DATE_TIME,
    SEQ_NO,
    MESSAGE,
    STATE,
    REASON_CD,
    DISCOVERY_IND,
    TRANNAME,
    ABENDCODE,
    PROGRAMNAME,
    TERMCODE
  )
  SELECT
    timestamp,
    ctime,
    parent_cid,
    parent_id,
    DATE_TIME,
    SEQ_NO,
    CONVERT(VARCHAR(16),MESSAGE),
    CONVERT(VARCHAR(40),STATE),
    REASON_CD,
    CONVERT(VARCHAR(4),DISCOVERY_IND),
    CONVERT(VARCHAR(8),field1),
    CONVERT(VARCHAR(16),field2),
    CONVERT(VARCHAR(8),field3),
    CONVERT(VARCHAR(4),field4)
   FROM #available_sjm 
  WHERE CHILD_OBJ_TYPE = 'TRAN'

  SET @msg = 'asisp_processStagedSJMs: calling asisp_processStagedFILEMESGs'
  EXEC asisp_checkpointmessage @msg, 0, @chkpt
  IF @debug = 1
    SELECT @chkpt AS chkpt, * FROM #available_sjm
  SELECT @chkpt = @chkpt + 1

  -- Call all of the detailed subjob message processors
  DECLARE @r INT

  EXEC @r = asisp_processStagedFILEMESGs
  IF @@error <> 0 or @r < 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedSJMs
    RETURN @r
  END
  ELSE
    DELETE StagedFILEMESG

  EXEC @r = asisp_processStagedTRANMESGs
  IF @@error <> 0 or @r < 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedSJMs
    RETURN @r
  END
  ELSE
    DELETE StagedTRANMESG

  EXEC @r = asisp_processStagedFMSGs
  IF @@error <> 0 or @r < 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedSJMs
    RETURN @r
  END
  ELSE
    DELETE StagedFMSG

  SET @msg = 'asisp_processStagedSJMs: Calling asisp_processStagedTRANDISCs'
  EXEC asisp_checkpointmessage @msg, 0, @chkpt
  IF @debug = 1
    SELECT @chkpt AS chkpt, * FROM StagedTRANDISC
  SELECT @chkpt = @chkpt + 1
    
  EXEC @r = asisp_processStagedTRANDISCs @debug = @debug
  IF @@error <> 0 or @r < 0
  BEGIN
    ROLLBACK TRANSACTION asisp_processStagedSJMs
    RETURN @r
  END
  ELSE
    DELETE StagedTRANDISC

  -- Mark events as processed
  INSERT ProcessedStagedEvent(evt_timestamp, ptime) 
    SELECT timestamp, getdate() FROM #processed_sjm
  EXEC asisp_checkpointmessage 'asisp_processStagedSJMs', 0, @chkpt
  SELECT @chkpt = @chkpt + 1

  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRANSACTION asisp_processStagedSJMs

  RETURN @num_events_processed
GO
