-- $Header:   //ARCHIVE/Archive/DB/SQL/asisp_repair_missing_lob_link.sqv   1.1   Mar 07 2001 12:14:40   BRENT  $
--
-- asisp_repair_missing_lob_link.sql
--

USE Object
GO


---------------------------------------------------------------------------------------------------
-- asisp_repair_missing_lob_link
PRINT 'Creating procedure asisp_repair_missing_lob_link'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_repair_missing_lob_link' AND type = 'P')
  DROP PROCEDURE asisp_repair_missing_lob_link
GO

-- $Archive:   //ARCHIVE/Archive/DB/SQL/asisp_repair_missing_lob_link.sqv  $
-- $Revision:   1.1  $
-- $Date:   Mar 07 2001 12:14:40  $
-- $Author:   BRENT  $
-- Finds all LOBs that have undeleted LOBL links to non-LOB objects which do not have corresponding
-- rows in lob_link table. These must be present for child events on LOBs to work properly.
CREATE PROCEDURE asisp_repair_missing_lob_link
AS
  DECLARE @starttime DATETIME
  SET @starttime = CURRENT_TIMESTAMP
  INSERT INTO MissingLOBLinkLog(starttime) VALUES ( @starttime )

  SET NOCOUNT ON
  DECLARE missingllc CURSOR FOR
    SELECT DISTINCT l.src_id, l.dst_cid, l.dst_id, l.ctime
    FROM link AS l 
    WHERE l.link_type = 'LOBL' AND l.dst_cid <> 'LOB' AND l.deleted = 0
      AND NOT EXISTS
      (
        SELECT * FROM lob_link AS ll WHERE ll.dst_id = l.src_id AND ll.phy_cid = l.dst_cid AND ll.phy_id = l.dst_id
      )
  OPEN missingllc
  DECLARE @src_id ObjID, @dst_cid ClassID, @dst_id ObjID, @ctime DATETIME, @num_repairs INT
  SET @num_repairs = 0
  FETCH NEXT FROM missingllc INTO @src_id, @dst_cid, @dst_id, @ctime
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    INSERT INTO MissingLOBLinkLogDetail(starttime, lob_id, phy_cid, phy_id, link_ctime)
      VALUES ( @starttime, @src_id, @dst_cid, @dst_id, @ctime )
    EXEC asisp_sync_lob_link @src_id
    SET @num_repairs = @num_repairs + 1
    FETCH NEXT FROM missingllc INTO @src_id, @dst_cid, @dst_id, @ctime
  END
  DEALLOCATE missingllc

  UPDATE MissingLOBLinkLog SET num_repairs = @num_repairs, endtime = CURRENT_TIMESTAMP
  WHERE starttime = @starttime
GO
