-- $Header:   //ARCHIVE/Archive/DB/SQL/asisp_sync_lob_link.sqv   1.0   Mar 07 2001 11:31:58   BRENT  $
--
-- asisp_sync_lob_link.sql

USE Object
GO


---------------------------------------------------------------------------------------------------
-- asisp_sync_lob_link
PRINT 'Creating procedure asisp_sync_lob_link'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_sync_lob_link' AND type = 'P')
  DROP PROCEDURE asisp_sync_lob_link
GO

-- $Archive:   //ARCHIVE/Archive/DB/SQL/asisp_sync_lob_link.sqv  $
-- $Revision:   1.0  $
-- $Date:   Mar 07 2001 11:31:58  $
-- $Author:   BRENT  $
-- Resynchronizes the lob_link table for the specified LOB.
-- lob_link records for each LOB all descendent LOBs and the objects to which they are logically linked.
create procedure asisp_sync_lob_link
  @lob_id ObjID
as
  SET NOCOUNT ON

  -- Standard transaction logic
  declare @tcount int
  select @tcount = @@trancount
  if @tcount = 0
    begin tran asisp_sync_lob_link
  else
    save tran asisp_sync_lob_link

  DECLARE @linked_cid ClassID, @linked_id ObjID
  SELECT @linked_cid = dst_cid, @linked_id = dst_id
  FROM link 
  WHERE src_cid = 'LOB' AND src_id = @lob_id AND deleted = 0
  IF @linked_id IS NULL
    SET @linked_id = 0

  declare @r int
  exec @r = asisp_init_lob_link @lob_id, @linked_cid, @linked_id
  if @r < 0
  begin
    rollback tran asisp_sync_lob_link
  end
  declare @gen int
  select @gen = 1
  -- Get rid of all previous entries where this LOB is on the src side
  delete lob_link where src_id = @lob_id
  if @@error <> 0
  begin
    rollback tran asisp_sync_lob_link
  end
  -- Repopulate lob_link starting with the children of this LOB
  insert lob_link(src_id, dst_id, gen, phy_cid, phy_id)
    select @lob_id, dst_id, @gen, null, 0
    from LOB_LOB_LOBC
    where src_id = @lob_id
  while @@rowcount > 0
  begin
    select @gen = @gen + 1
    insert lob_link(src_id, dst_id, gen, phy_cid, phy_id)
      select @lob_id, lll.dst_id, @gen, null, 0
      from lob_link ll, LOB_LOB_LOBC lll
      where
        ll.src_id = @lob_id and gen = @gen - 1
        and lll.src_id = ll.dst_id
  end
  -- Follow logical links
  update lob_link 
    set phy_cid = l.dst_cid, phy_id = l.dst_id
    from link l
    where 
      lob_link.src_id = @lob_id
      and link_type = 'LOBL'
      and src_cid = 'LOB'
      and l.src_id = lob_link.dst_id
      and deleted = 0
  while @@rowcount > 0
  begin
  update lob_link 
    set 
      phy_cid = l.dst_cid,
      phy_id = l.dst_id
    from link l
    where 
      lob_link.src_id = @lob_id
      and phy_cid = 'LOB' 
      and link_type = 'LOBL' 
      and l.src_cid = 'LOB'
      and l.src_id = lob_link.phy_id 
      and deleted = 0
  end
  if @tcount = 0
    commit tran asisp_sync_lob_link
GO
