-- $Header:   //ARCHIVE/Archive/DB/SQL/tD_RODMManagedLink.sqv   1.0   Mar 13 2001 11:10:56   BRENT  $
--
-- tD_RODMManagedLink.sql
--

USE RODM
GO

-- tD_RODMManagedLink
PRINT 'Creating trigger tD_RODMManagedLink'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tD_RODMManagedLink' AND type = 'TR')
  DROP TRIGGER tD_RODMManagedLink
GO

-- $Archive:   //ARCHIVE/Archive/DB/SQL/tD_RODMManagedLink.sqv  $
-- $Revision:   1.0  $
-- $Date:   Mar 13 2001 11:10:56  $
-- $Author:   BRENT  $
CREATE TRIGGER tD_RODMManagedLink ON RODMManagedLink FOR DELETE
AS
  DECLARE @link_type VARCHAR(4)
  DECLARE @src_cid ClassID
  DECLARE @dst_cid ClassID
  DECLARE @sql VARCHAR(256)

  IF EXISTS
  (
    SELECT *
    FROM deleted
    WHERE link_type <> '' AND src_cid <> '' AND dst_cid <> ''
  )
  BEGIN
    -- Create temp table with the source and destination object ids. 
    -- This is used in the dynamic sql that follows since the 'deleted'
    -- pseudo-table cannot be referenced in the dynamic sql.
    SELECT link_type, src_cid, src_id, dst_cid, dst_id 
    INTO #old
    FROM deleted
    WHERE link_type <> '' AND src_cid <> '' AND dst_cid <> ''

    -- Insert LinkDeleted notifications
    INSERT Object..PendingNotification(type, fields, detail)
      SELECT
        e.type,
        lt.link_type + '|' + 
        CONVERT(VARCHAR(9), ISNULL(src.cno, 0))  + '|' +
        CONVERT(VARCHAR(9), ISNULL(d.src_id, 0)) + '|' +
        CONVERT(VARCHAR(9), ISNULL(dst.cno, 0))  + '|' +
        CONVERT(VARCHAR(9), ISNULL(d.dst_id, 0)) + '|' +
        CONVERT(VARCHAR(9), ISNULL(lt.flags, 0)),
        ''
      FROM
        Object..EventType AS e,
        #old AS d
        JOIN obj_class AS src ON src.cid = d.src_cid
        JOIN obj_class AS dst ON dst.cid = d.dst_cid
        JOIN link_type AS lt ON lt.link_type = d.link_type
      WHERE
        e.name = 'LinkDeleted'

    -- Delete the individual link table records
    DECLARE linkCursor CURSOR FOR 
      SELECT DISTINCT link_type, src_cid, dst_cid
      FROM #old
    OPEN linkCursor
    FETCH NEXT FROM linkCursor INTO @link_type, @src_cid, @dst_cid
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      -- Insert link records
      DECLARE @table sysname
      SET @table = RTRIM(@src_cid) + '_' + RTRIM(@dst_cid) + '_' + @link_type
      IF NOT EXISTS (SELECT * FROM Object..sysobjects WHERE name = @table AND type = 'U')
      BEGIN
        INSERT INTO RODMLoadError(error) SELECT 'Skipping attempt to delete from nonexistent link table ' + @table
        PRINT 'Skipping attempt to delete from nonexistent link table ' + @table
      END
      ELSE
      BEGIN
        SET @sql = 'DELETE l FROM Object..' + @table + ' AS l ' +
                   'JOIN #old AS o ON o.src_id = l.src_id AND o.dst_id = l.dst_id'
        PRINT 'tD_RODMManagedLink: ' + @sql
        EXEC(@sql)
      END
      FETCH NEXT FROM linkCursor INTO @link_type, @src_cid, @dst_cid
    END
    DEALLOCATE linkCursor
  END
GO
