-- $Header:   //ARCHIVE/Archive/DB/SQL/tI_RODMManagedLink.sqv   1.2   Mar 13 2001 11:10:34   BRENT  $
--
-- tI_RODMManagedLink.sql
--

USE RODM
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'RODMLoadError' AND type = 'U')
BEGIN
  PRINT 'Creating table RODMLoadError'
  EXEC("
  CREATE TABLE RODMLoadError
  (
    ctime    DATETIME NOT NULL     DEFAULT CURRENT_TIMESTAMP,
    rowno    INT      NOT NULL     IDENTITY(1,1),
    error    VARCHAR(255) NOT NULL
  )
  ALTER TABLE RODMLoadError ADD CONSTRAINT RODMLoadError_PK PRIMARY KEY (ctime, rowno)
  ")
END

-- tI_RODMManagedLink
PRINT 'Creating trigger tI_RODMManagedLink'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tI_RODMManagedLink' AND type = 'TR')
  DROP TRIGGER tI_RODMManagedLink
GO

-- $Archive:   //ARCHIVE/Archive/DB/SQL/tI_RODMManagedLink.sqv  $
-- $Revision:   1.2  $
-- $Date:   Mar 13 2001 11:10:34  $
-- $Author:   BRENT  $
-- Insert trigger for RODM..RODMManagedLink table. This will cascade the inserts
-- to the appropriate Object link tables.
CREATE TRIGGER tI_RODMManagedLink ON RODMManagedLink FOR INSERT
AS
  DECLARE @link_type VARCHAR(4)
  DECLARE @src_cid ClassID
  DECLARE @dst_cid ClassID
  DECLARE @sql VARCHAR(500)

  IF EXISTS 
  (
    SELECT * 
    FROM inserted 
    WHERE link_type <> '' AND src_cid <> '' AND dst_cid <> ''
  )
  BEGIN
    -- Create temp table with the source and destination object ids. 
    -- This is used in the dynamic sql that follows since the 'inserted'
    -- pseudo-table cannot be referenced in the dynamic sql.
    SELECT link_type, src_cid, src_id, dst_cid, dst_id 
    INTO #new
    FROM inserted 
    WHERE link_type <> '' AND src_cid <> '' AND dst_cid <> ''

    -- Insert LinkCreated notifications
    INSERT Object..PendingNotification(type, fields, detail)
      SELECT
        e.type,
        lt.link_type + '|' + 
        CONVERT(VARCHAR(9), ISNULL(src.cno, 0))  + '|' +
        CONVERT(VARCHAR(9), ISNULL(i.src_id, 0)) + '|' +
        CONVERT(VARCHAR(9), ISNULL(dst.cno, 0))  + '|' +
        CONVERT(VARCHAR(9), ISNULL(i.dst_id, 0)) + '|' +
        CONVERT(VARCHAR(9), ISNULL(lt.flags, 0)),
        ''
      FROM
        Object..EventType AS e,
        #new AS i
        JOIN obj_class AS src ON src.cid = i.src_cid
        JOIN obj_class AS dst ON dst.cid = i.dst_cid
        JOIN link_type AS lt ON lt.link_type = i.link_type
      WHERE
        e.name = 'LinkCreated'

    -- Insert rows into the individual link tables
    DECLARE linkCursor CURSOR FOR 
      SELECT DISTINCT link_type, src_cid, dst_cid 
      FROM #new
    OPEN linkCursor
    FETCH NEXT FROM linkCursor INTO @link_type, @src_cid, @dst_cid
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      -- Insert link records
      DECLARE @table sysname
      SET @table = RTRIM(@src_cid) + '_' + RTRIM(@dst_cid) + '_' + @link_type
      IF NOT EXISTS (SELECT * FROM Object..sysobjects WHERE name = @table AND type = 'U')
      BEGIN
        INSERT INTO RODMLoadError(error) SELECT 'Skipping attempt to insert into nonexistent link table ' + @table
        PRINT 'Skipping attempt to insert into nonexistent link table ' + @table
      END
      ELSE
      BEGIN
        SET @sql = 'INSERT Object..' + @table + ' (src_id,dst_id) ' +
                   'SELECT src_id, dst_id FROM #new AS new ' +
                   'WHERE src_cid=''' + @src_cid + ''' AND dst_cid=''' + @dst_cid + ''' AND link_type=''' + @link_type + ''' ' + 
                   'AND NOT EXISTS (SELECT * FROM Object..' + RTRIM(@src_cid) + '_' + RTRIM(@dst_cid) + '_' + @link_type + ' AS old ' +
                   'WHERE new.src_id = old.src_id AND new.dst_id = old.dst_id)'
        PRINT 'tI_RODMManagedLink: ' + @sql
        EXEC(@sql)
      END

      FETCH NEXT FROM linkCursor INTO @link_type, @src_cid, @dst_cid
    END
    DEALLOCATE linkCursor
  END
GO
