-- $Header:   //ARCHIVE/Archive/DB/sql/tU_LastMessageAlertStatePri.sqv   1.2   Apr 05 2001 12:55:48   BRENT  $
--
-- tU_LastMessageAlertStatePri.sql
--

USE Object
GO

---------------------------------------------------------------------------------------------------
-- tU_LastMessageAlertStatePri 
PRINT 'Creating trigger tU_LastMessageAlertStatePri '
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_LastMessageAlertStatePri ' AND type = 'TR')
  DROP TRIGGER tU_LastMessageAlertStatePri
GO

-- $Archive:   //ARCHIVE/Archive/DB/sql/tU_LastMessageAlertStatePri.sqv  $
-- $Revision:   1.2  $
-- $Date:   Apr 05 2001 12:55:48  $
-- $Author:   BRENT  $
-- Update trigger for LastMessageAlertStatePri table.
-- When the row for a particular object is updated, the trigger checks to see whether any rows
-- are having AlertState or Priority being updated. If any row is going from a non-green AlertState
-- to another non-green AlertState, then if either the AlertState or the Priority have changed, we
-- have to cleanup any previous message-based childevent with the old AlertState/Priority. This is
-- because a message-based childevent should reflect the most recent non-green message.
--
-- Note that there is a kludge to handle the case where the new message has the same alert state
-- as the old but a different priority. In this case, the ASIPAgent will not generate a new child
-- event since the alert states are the same. This procedure catches this condition and generates
-- a new child event with the correct priority.
CREATE TRIGGER tU_LastMessageAlertStatePri ON LastMessageAlertStatePri FOR UPDATE
AS
  IF UPDATE(AlertState) OR UPDATE(Priority)
  BEGIN
    IF EXISTS
    (
      SELECT *
      FROM
        inserted AS i
        JOIN deleted AS d ON d.cid = i.cid AND d.id = i.id
      WHERE
        d.AlertState > 1
        AND i.AlertState > 1
        AND
        (
          d.AlertState <> i.AlertState
          OR
          d.Priority <> i.Priority
        )
    )
    BEGIN
      DECLARE clearc CURSOR FOR
        SELECT d.cid, d.id, d.AlertState, d.Priority, d.mesg_id, i.AlertState, i.Priority, i.mesg_id
        FROM
          inserted AS i
          JOIN deleted AS d ON d.cid = i.cid AND d.id = i.id
        WHERE
          d.AlertState > 1
          AND i.AlertState > 1
          AND
          (
            d.AlertState <> i.AlertState
            OR
            d.Priority <> i.Priority
          )
      OPEN clearc
      DECLARE @cid ClassID, @id ObjID,
        @old_AlertState INT, @old_Priority INT, @old_mesg_id ObjID,
        @new_AlertState INT, @new_Priority INT, @new_mesg_id ObjID
      FETCH NEXT FROM clearc INTO @cid, @id, @old_AlertState, @old_Priority, @old_mesg_id, @new_AlertState, @new_Priority, @new_mesg_id
      WHILE @@FETCH_STATUS <> -1
      BEGIN
        -- If the AlertState stays the same but the priority is different we have to generate a child event
        -- at the new priority.
        DECLARE @regen TINYINT
        SET @regen = 0
        IF @old_AlertState = @new_AlertState AND @old_Priority <> @new_Priority
          SET @regen = 1

        -- We have to check the priority of the object first because if it is not set to
        -- InheritFromEvent (6) then the old and new child events will be at the object's
        -- priority instead.
        DECLARE @cname VARCHAR(24), @id_str VARCHAR(9), @obj_Priority INT
        SELECT @cname = cname, @id_str = CAST(@id AS VARCHAR) FROM obj_class WHERE cid = @cid
        EXEC("
        DECLARE prilookupc CURSOR FOR SELECT _PriorityID FROM " + @cname + "_V WHERE id = " + @id_str
        )
        OPEN prilookupc
        FETCH NEXT FROM prilookupc INTO @obj_Priority
        DEALLOCATE prilookupc
        IF @obj_Priority <> 6 -- InheritFromEvent
        BEGIN
          SET @old_Priority = @obj_Priority
          SET @new_Priority = @obj_Priority
        END

        -- Clear old child event
        DECLARE @EventDetail VARCHAR(255)
        SELECT @EventDetail = 'MESSAGE ' + CAST(@old_mesg_id AS VARCHAR) + ' SUPERCEEDED BY MESSAGE ' + CAST(@new_mesg_id AS VARCHAR)
        EXEC asisp_generatechildevent
          @src_cid    = NULL,
          @src_id     = NULL,
          @dst_cid    = @cid,
          @dst_id     = @id,
          @EvtCID     = 'MESG',
          @EvtDetail  = @EventDetail,
          @EvtCount   = 0,
          @Direction  = -1,
          @Priority   = @old_Priority,
          @AlertState = @old_AlertState
        IF @regen = 1
        BEGIN
          SELECT @EventDetail = _MESSAGE FROM Message_V WHERE id = @new_mesg_id
          EXEC asisp_generatechildevent
            @src_cid    = NULL,
            @src_id     = NULL,
            @dst_cid    = @cid,
            @dst_id     = @id,
            @EvtCID     = 'MESG',
            @EvtDetail  = @EventDetail,
            @EvtCount   = 0,
            @Direction  = 1,
            @Priority   = @new_Priority,
            @AlertState = @new_AlertState
        END
        FETCH NEXT FROM clearc INTO @cid, @id, @old_AlertState, @old_Priority, @old_mesg_id, @new_AlertState, @new_Priority, @new_mesg_id
      END
      DEALLOCATE clearc
    END
  END
GO
